/*
This file contains the global variable declarations, function declarations, 
and some definitions used in many of the routines. The global variables are 
defined in the file latticeeasy.cpp.
*/

#ifndef _LATTICEEASYHEADER_
#define _LATTICEEASYHEADER_

#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>

const float pi = (float)(2.*asin(1.));
inline float pw2(float x) {return x*x;} // Useful macro for squaring floats

/////////////////////////////////INCLUDE ADJUSTABLE PARAMETERS///////////////////
#include "parameters.h"

/////////////////////////////////GLOBAL DYNAMIC VARIABLES////////////////////////
extern float f[nflds][N][N][N],fd[nflds][N][N][N]; // Field values and derivatives
extern float t,t0; // Current time and initial time (t0=0 unless the run is a continuation of a previous one)
extern float a,ad,ad2,aterm; // Scale factor and its derivatives (aterm is a combination of the others used in the equations of motion)
extern float hubble_init; // Initial value of the Hubble constant
extern int run_number; // 0 for a first run, 1 for a continuation of a "0" run, etc.. Stored in the grid image (see checkpoint() function).

/////////////////////////////////NON-ADJUSTABLE VARIABLES////////////////////////
// Derived run parameters
const float dx=L/(float)N; // Distance between adjacent gridpoints
const int gridsize=N*N*N; // Number of spatial points in the grid

/////////////////////////////////INCLUDE SPECIFIC MODEL//////////////////////////
#include "model.h"

/////////////////////////////////FUNCTION DECLARATIONS///////////////////////////
// initialize.cpp
void initialize(); // Set initial parameters and field values
// evolution.cpp
float gradient_energy(int fld); // Calculate the gradient energy, <|Grad(f)|^2>=<-f Lapl(f)>, of a field
void evolve_scale(float d); // Calculate the scale factor and its derivatives
void evolve_fields(float d); // Advance the field values and scale factor using the first derivatives
void evolve_derivs(float d); // Calculate second derivatives of fields and use them to advance first derivatives. Also calls evolve_scale().
// output.cpp
void output_parameters(); // Output information about the run parameters
void save(int force); // Calculate and save quantities (means, variances, spectra, etc.)
// ffteasy.cpp
void fftrn(float f[], float fnyquist[], int ndims, int size[], int forward); // Do a Fourier transform of an ndims dimensional array of real numbers

#endif // End of conditional for definition of _LATTICEEASYHEADER_ macro






